# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""Function dealing with (crypto) keys"""

from nova import crypto
from nova import db
from nova import exception
from nova import flags
from nova import log as logging


FLAGS = flags.FLAGS

LOG = logging.getLogger("nova.keys")


def _gen_key(context, user_id, key_name):
    """Generate a key

    This is a module level method because it is slow and we need to defer
    it into a process pool."""
    # NOTE(vish): generating key pair is slow so check for legal
    #             creation before creating key_pair
    try:
        db.key_pair_get(context, user_id, name=key_name)
        raise exception.Duplicate(_("The key_pair %s already exists")
                                  % key_name)
    except exception.NotFound:
        pass
    private_key, public_key, fingerprint = crypto.generate_key_pair()
    key = {}
    key['user_id'] = user_id
    key['name'] = key_name
    key['public_key'] = public_key
    key['fingerprint'] = fingerprint
    created = db.key_pair_create(context, key)
    return {'private_key': private_key, 'key': created}


class API(object):
    """Although keys is currently an in-process function, this abstraction
    supports the idea that it may not be in future, so that all the APIs
    look the same (volumes, compute, keys etc)."""
    def get_all_keys(self, context):
        key_pairs = db.key_pair_get_all_by_user(context, context.user_id)

        result = []
        for key_pair in key_pairs:
            # filter out the vpn keys
            suffix = FLAGS.vpn_key_suffix
            if context.is_admin or not key_pair['name'].endswith(suffix):
                result.append(key_pair)

        return result

    def get_key_by_name(self, context, key_name):
        return db.key_pair_get(context, context.user_id, name=key_name)

    def get_key_by_id(self, context, key_id):
        return db.key_pair_get(context, context.user_id, id=key_id)

    def create_key_pair(self, context, key_name):
        LOG.audit(_("Create key pair %s"), key_name, context=context)
        return _gen_key(context, context.user_id, key_name)
        # TODO(vish): when context is no longer an object, pass it here

    def delete_key_pair(self, context, key):
        key_name = key['name']
        LOG.audit(_("Delete key pair %s"), key_name, context=context)
        try:
            db.key_pair_destroy(context, context.user_id, name=key_name)
        except exception.NotFound:
            # aws returns true even if the key doesn't exist
            pass
        return True
